ARCH_CHROOT = '/usr/bin/arch-chroot'
BLKID = '/usr/bin/blkid'
DEBOOTSTRAP = '/usr/bin/debootstrap'
MKFS_EXT4 = '/usr/bin/mkfs.ext4'
MKSWAP = '/usr/bin/mkswap'
MOUNT = '/usr/bin/mount'
PACSTRAP = '/usr/bin/pacstrap'
QEMU_NBD = '/usr/bin/qemu-nbd'
SED = '/usr/bin/sed'
SGDISK = '/usr/bin/sgdisk'
SWAPON = '/usr/bin/swapon'
SWAPOFF = '/usr/bin/swapoff'
TUNE2FS = '/usr/bin/tune2fs'
UMOUNT = '/usr/bin/umount'
